#!/usr/bin/python3.6
import requests
import os
from os.path import expanduser
from dotenv import load_dotenv

env_path = expanduser("~/.env")
load_dotenv(dotenv_path=env_path)


def telegram_bot_sendtext(bot_message):

    token = os.getenv("API_TOKEN")
    bot_id = os.getenv("BOT_ID")

    send_text = (
        "https://api.telegram.org/bot"
        + token
        + "/sendMessage?chat_id="
        + bot_id
        + "&parse_mode=Markdown&text="
        + bot_message
    )

    response = requests.get(send_text)

    return response.json()



#!/usr/bin/python3.6
import tweepy
from scibot.telebot import telegram_bot_sendtext
from dotenv import load_dotenv
from os.path import expanduser
import os

env_path = expanduser("~/.env")
load_dotenv(dotenv_path=env_path, override=True)

# Setup API:
def twitter_setup():
    """
    Setup Twitter connection for a developer account
    Returns: tweepy.API object

    """
    # Authenticate and access using keys:
    auth = tweepy.OAuthHandler(os.getenv("CONSUMER_KEY"), os.getenv("CONSUMER_SECRET"))
    auth.set_access_token(os.getenv("ACCESS_TOKEN"), os.getenv("ACCESS_SECRET"))

    # Return API access:
    api = tweepy.API(auth, wait_on_rate_limit=True, wait_on_rate_limit_notify=True)
    return api

api = twitter_setup()

banned_profiles = ['nydancesafe']

class MyStreamListener(tweepy.StreamListener):

    def on_status(self, status):

        if hasattr(status, "retweeted_status"):  # Check if Retweet
#             try:
#                 print('rt',status['retweeted_status']['extended_tweet']["full_text"])
#             except AttributeError:
            telegram_bot_sendtext(f" check if retweet:, {status.retweeted_status.text}")
            if "constellation" not in status.retweeted_status.text.lower():
                pass
        else:
            try:
                ## catch nesting
                if status.user.screen_name in banned_profiles or status.in_reply_to_screen_name:
                    pass
                replied_to=status.in_reply_to_screen_name
                answer_user=status.user.screen_name
                answer_id=status.id
                answer_user_id = status.user.id
                ## ignore replies that by default contain mention
                in_reply_to_status_id=status.in_reply_to_status_id
                in_reply_to_user_id=status.in_reply_to_user_id

                telegram_bot_sendtext(f"{replied_to}, 'nesting', {in_reply_to_user_id}, 'replied to', {replied_to}, 'message', {status.text}")

            except AttributeError:

                replied_to=status.in_reply_to_screen_name
                answer_user=status.user.screen_name
                answer_user_id = status.user.id
                answer_id=status.id
                in_reply_to_status_id=status.in_reply_to_status_id
                in_reply_to_user_id=status.in_reply_to_user_id

                telegram_bot_sendtext(f"ATRIB ERROR: {replied_to}, 'nesting', {in_reply_to_user_id}, 'replied to', {replied_to}, 'message', {status.text}")

            update_status = f""" #ConstellationsFest live RT. From 16-24 NOV:

https://twitter.com/{answer_user}/status/{answer_id}
             """

            # don't reply to yourself!!
            self_ids=[1319577341056733184, 1118874276961116162]
            if status.user.id not in self_ids:

                api.update_status(update_status,
                auto_populate_reply_metadata=True)


    def on_error(self, status):
        telegram_bot_sendtext(f"ERROR with: {status}")

def listen_stream_and_rt(keywords_list):
    api = twitter_setup()
    myStreamListener = MyStreamListener()
    try:
        myStream = tweepy.Stream(auth=api.auth, listener=myStreamListener)
        myStream.filter(track=keywords_list, is_async=True)
    except Exception as ex:
        telegram_bot_sendtext(f"ERROR with: {ex}")
        pass



#!/usr/bin/python3.6
import json
import os
import sys
import re
import time
from os.path import expanduser
from random import randint
import calendar
import tweepy
from bs4 import BeautifulSoup
from dotenv import load_dotenv
from scibot.streamer import listen_stream_and_rt
from scibot.telebot import telegram_bot_sendtext
from scibot.tools import (
    logger,
    Settings,
    insert_hashtag,
    shorten_text,
    compose_message,
    is_in_logfile,
    write_to_logfile,
    scheduled_job,
)

env_path = expanduser("~/.env")
load_dotenv(dotenv_path=env_path, override=True)


def main():
    """
    Main function of scibot

    """

    logger.info("\n### sciBot started ###\n\n")
    if len(sys.argv) > 1:
        try:
            check_json_exists(
                Settings.users_json_file,
                {"test": {"follower": False, "interactions": 0}},
            )
            check_json_exists(Settings.faved_tweets_output_file, {"test": {}})
            check_json_exists(
                Settings.posted_retweets_output_file,
                {"test": {}},
            )
            check_json_exists(
                Settings.posted_urls_output_file,
                {"test": {}},
            )

            if sys.argv[1].lower() == "rss":
                read_rss_and_tweet()
            elif sys.argv[1].lower() == "str":
                listen_stream_and_rt(['#ConstellationsFest', '#ConstellationFest'])
            elif sys.argv[1].lower() == "rtg":
                search_and_retweet("global_search")
            elif sys.argv[1].lower() == "glv":
                search_and_retweet("give_love")
            elif sys.argv[1].lower() == "rtl":
                search_and_retweet("list_search")
            elif sys.argv[1].lower() == "rto":
                retweet_old_own()
            elif sys.argv[1].lower() == "sch":
                listen_stream_and_rt(['#ConstellationsFest', '#ConstellationFest'])
                scheduled_job(read_rss_and_tweet, retweet_old_own, search_and_retweet)


        except Exception as e:
            logger.exception(e, exc_info=True)
            telegram_bot_sendtext(f"[Exception] {e}")

        except IOError as errno:
            logger.exception(f"[ERROR] IOError {errno}")
            telegram_bot_sendtext(f"[ERROR] IOError {errno}")

    else:
        display_help()
    logger.info("\n\n### sciBot finished ###")


# Setup API:
def twitter_setup():
    """
    Setup Twitter connection for a developer account
    Returns: tweepy.API object

    """
    # Authenticate and access using keys:
    auth = tweepy.OAuthHandler(os.getenv("CONSUMER_KEY"), os.getenv("CONSUMER_SECRET"))
    auth.set_access_token(os.getenv("ACCESS_TOKEN"), os.getenv("ACCESS_SECRET"))

    # Return API access:
    api = tweepy.API(auth, wait_on_rate_limit=True)
    return api


def check_json_exists(file_path: os.path, init_dict: dict) -> None:
    """

    Create a folder and json file if does not exists

    Args:
        file_path: Log files file path
        init_dict: Dictionary to initiate a json file

    Returns: None

    """

    if not os.path.exists(os.path.dirname(os.path.abspath(file_path))):
        os.makedirs(file_path)

    if not os.path.isfile(file_path):
        with open(file_path, "w") as json_file:
            json.dump(init_dict, json_file, indent=4)


def get_followers_list() -> list:
    """
    Read json file of followers from Settings.users_json_file
    Returns: List of followers

    """

    with open(Settings.users_json_file, "r") as json_file:
        users_dic = json.load(json_file)
    return [x for x in users_dic if users_dic[x]["follower"] is True]


def update_thread(text: str, tweet: tweepy.Status, api: tweepy.API) -> tweepy.Status:
    """
    Add a tweet to a initiated thread
    Args:
        text: text to add to tweet as thread
        tweet: tweepy status to add reply to
        api: tweepy.API object

    Returns: post a reply to a tweet

    """
    return api.update_status(
        status=text, in_reply_to_status_id=tweet.id, auto_populate_reply_metadata=True
    )


def post_thread(dict_one_pub: dict, maxlength: int, count: int = 1) -> int:
    """
    Initiate and post a thread of tweets
    Args:
        dict_one_pub: dictionary object with processed publication item
        maxlength: length of the message to post (max tweet is 280 char)
        count: count of replies on the thread

    Returns: tweet id of the first tweet of thread

    """
    api = twitter_setup()
    original_tweet = api.update_status(status=compose_message(dict_one_pub))
    telegram_bot_sendtext(f"Posting thread:, twitter.com/drugscibot/status/{original_tweet.id}")

    text = dict_one_pub["abstract"]
    max_len =  round(len(text)/3)
    if max_len < 240:
        maxlength = max_len


    for index in range(0, len(text), maxlength):
        if count < 4:
            count += 1
            time.sleep(2)
            thread_message = (
                insert_hashtag(text[index : index + maxlength]) + f"... {count}/5"
            )
            if count == 2:
                reply1_tweet = update_thread(thread_message, original_tweet, api)
            if count == 3:
                reply2_tweet = update_thread(thread_message, reply1_tweet, api)
            if count == 4:
                reply3_tweet = update_thread(thread_message, reply2_tweet, api)

    time.sleep(2)
    count += 1
    last_msg = shorten_text(dict_one_pub["pub_date"] + " " + dict_one_pub["author-s"], 250) + f" {count}/{count}"

    update_thread(last_msg, reply3_tweet, api)

    return original_tweet.id

def return_doi_str(article):
    """return doi link if exists"""
    title_search = re.search('(DOI:<a href=")(.*)(">)', str(article))
    if title_search:
        return title_search.group(2)
    else:
        return article.link


def make_literature_dict(feed: list) -> dict:
    """
    filter publications from an RSS feed having an abstract, parse html abstract as plane string
    Args:
        feed: list of RSS feed items

    Returns: dictionary of processed publications

    """

    dict_publications = {}

    for item in feed:
        if hasattr(item, "content") and not 'No abstract' in item.description:

            authors_list = [x["name"] for x in item.authors]

            dict_publications[item.id] = {
                "title": item.title,
                "abstract": BeautifulSoup(item.content[0].value, "html.parser")
                .get_text()
                .split("ABSTRACT")[1],
                "link": return_doi_str(item),
                "description": item.description,
                "pub_date": f"Date: {calendar.month_name[item.published_parsed.tm_mon]} {item.published_parsed.tm_year}",
                "author-s": f"Authors:  {', '.join(authors_list)}" if len(authors_list) >1 else  f"Author:  {', '.join(authors_list)}",

            }
    return dict_publications


def read_rss_and_tweet() -> None:
    """

    Read RSS objects and tweet one calling the post thread function
    Returns: None, updates log file with the posted article id

    """
    dict_publications = make_literature_dict(Settings.combined_feed)

    with open(Settings.posted_urls_output_file, "r") as jsonFile:
        article_log = json.load(jsonFile)

    if all(item in article_log.keys() for item in dict_publications.keys()):

        telegram_bot_sendtext("rss empty trying older articles")
        dict_publications = make_literature_dict(Settings.feed_older_literature)

    for article in sorted(dict_publications.keys(), reverse=True):

        if not is_in_logfile(article, Settings.posted_urls_output_file):
            try:
                article_log[article] = {
                    "count": 1,
                    "tweet_id": post_thread(dict_publications[article], 240),
                }

                write_to_logfile(article_log, Settings.posted_urls_output_file)
                break
            except tweepy.TweepError as e:
                logger.error(f"RSS error, possible duplicate {e}, {article}")
                write_to_logfile(article_log, Settings.posted_urls_output_file)
                continue


def json_add_new_friend(user_id: str) -> None:
    """
    add user friends to the interactions json file
    Args:
        user_id: user id to add to the interactions file

    Returns: None, updates interaction file

    """

    with open(Settings.users_json_file, "r") as json_file:
        users_dic = json.load(json_file)
    if user_id not in users_dic:
        users_dic[user_id] = {"follower": True, "interactions": 1}
    else:
        users_dic[user_id]["follower"] = True

    with open(Settings.users_json_file, "w") as json_file:
        json.dump(users_dic, json_file, indent=4)


def post_tweet(message: str) -> None:
    """
    Post tweet message to account.
    Args:
        message: Message to post on Twitter

    Returns: None

    """
    try:
        twitter_api = twitter_setup()
        logger.info(f"post_tweet():{message}")
        twitter_api.update_status(status=message)
    except tweepy.TweepError as e:
        logger.error(e)


def filter_repeated_tweets(result_search: list, flag: str) -> list:
    """

    Args:
        result_search:
        flag:

    Returns:

    """

    if flag == "give_love":
        out_file = Settings.faved_tweets_output_file
    else:
        out_file = Settings.posted_retweets_output_file

    unique_results = {}

    for status in result_search:
        if hasattr(status, "retweeted_status"):
            check_id = status.retweeted_status.id_str
        else:
            check_id = status.id_str

        if not is_in_logfile(check_id, out_file):
            unique_results[status.full_text] = status

    return [unique_results[x] for x in unique_results]


def json_add_user(user_id: str) -> None:
    """
    add user to the interactions json file
    Args:
        user_id: user id

    Returns: None

    """
    with open(Settings.users_json_file, "r") as json_file:
        users_dic = json.load(json_file)
    if user_id not in users_dic:
        users_dic[user_id] = {"follower": False, "interactions": 1}
    else:
        users_dic[user_id]["interactions"] += 1

    with open(Settings.users_json_file, "w") as json_file:
        json.dump(users_dic, json_file, indent=4)


def get_query() -> str:
    """
    Create Twitter search query with included words minus the
    excluded words.

    Returns:  string with the Twitter search query

    """
    include = " OR ".join(Settings.retweet_include_words)
    exclude = " -".join(Settings.retweet_exclude_words)
    exclude = "-" + exclude if exclude else ""
    return include + " " + exclude


def check_interactions(tweet: tweepy.Status) -> None:
    """
    check if previously interacted with a user
    Args:
        tweet:

    Returns:

    """

    if tweet.author.screen_name.lower() == "viewsondrugsbot":
        pass  # don't fav your self

    auth_id = tweet.author.id_str
    with open(Settings.users_json_file, "r") as json_file:
        users_dic = json.load(json_file)

        user_list = [
            users_dic[x]["interactions"]
            for x in users_dic
            if users_dic[x]["follower"] == False
        ]

        down_limit = round(sum(user_list) / len(user_list))

        if auth_id in users_dic:
            if users_dic[auth_id]["interactions"] >= down_limit:
                return True
            else:
                return False
        else:
            return False


def try_retweet(
    twitter_api: tweepy.API, tweet_text: str, in_tweet_id: str, self_followers: list
) -> None:
    """
    try to retweet, if already retweeted try next fom the list
    of recent tweets
    Args:
        twitter_api:
        tweet_text:
        in_tweet_id:
        self_followers:

    Returns:

    """

    tweet_id = find_simple_users(twitter_api, in_tweet_id, self_followers)

    if not is_in_logfile(in_tweet_id, Settings.posted_retweets_output_file):
        try:
            twitter_api.retweet(id=tweet_id)
            logger.info(f"Trying to rt {tweet_id}")
            write_to_logfile({in_tweet_id: {}}, Settings.posted_retweets_output_file)
            _status = twitter_api.get_status(tweet_id)
            json_add_user(_status.author.id_str)
            if tweet_id == in_tweet_id:
                id_mess = f"{tweet_id} original"
            else:
                id_mess = f"{tweet_id} from a nested profile"
            message_log = (
                "Retweeted and saved to file >  https://twitter.com/i/status/{}".format(
                    id_mess
                )
            )
            logger.info(message_log)
            telegram_bot_sendtext(message_log)
            return True
        except tweepy.TweepError as e:
            if e.api_code in Settings.IGNORE_ERRORS:
                write_to_logfile(
                    {in_tweet_id: {}}, Settings.posted_retweets_output_file
                )
                logger.exception(e)
                return False
            else:
                logger.error(e)
                return True
    else:
        logger.info(
            "Already retweeted {} (id {})".format(
                shorten_text(tweet_text, maxlength=140), tweet_id
            )
        )


def get_longest_text(status: tweepy.Status) -> str:
    """
    Get the text of a quoted status
    Args:
        status: tweepy.Status object

    Returns: text of the quoted tweet

    """
    if hasattr(status, "retweeted_status"):
        return status.retweeted_status.full_text
    else:
        return status.full_text


def find_simple_users(
    twitter_api: tweepy.API, tweet_id: str, followers_list: list
) -> int:
    """
    retweet/fav from users retweeting something interesting
    Args:
        twitter_api:
        tweet_id:
        followers_list:

    Returns: id of the retweeted/faved tweet

    """
    # get original retweeter:
    down_lev_tweet = twitter_api.get_status(tweet_id)

    if hasattr(down_lev_tweet, "retweeted_status"):
        retweeters = twitter_api.retweets(down_lev_tweet.retweeted_status.id_str)
    else:
        retweeters = twitter_api.retweets(tweet_id)

    future_friends = []
    for retweet in retweeters:

        if check_interactions(retweet):
            continue
        try:
            follows_friends_ratio = (
                retweet.author.followers_count / retweet.author.friends_count
            )
        except ZeroDivisionError:
            follows_friends_ratio = 0

        future_friends_dic = {
            "id_str": retweet.author.id_str,
            "friends": retweet.author.friends_count,
            "followers": retweet.author.followers_count,
            "follows_friends_ratio": follows_friends_ratio,
        }
        if future_friends_dic["friends"] > future_friends_dic["followers"]:
            future_friends.append(
                (
                    future_friends_dic["follows_friends_ratio"],
                    retweet.id_str,
                    future_friends_dic,
                )
            )
        else:
            future_friends.append(
                (future_friends_dic["followers"], retweet.id_str, future_friends_dic)
            )
    if future_friends:
        try:  # give prioroty to non followers of self
            min_friend = min(
                [x for x in future_friends if x[2]["id_str"] not in followers_list]
            )
            logger.info(
                f"try retweeting/fav https://twitter.com/i/status/{min_friend[1]} from potential friend profile: {min_friend[2]['id_str']} friends= {min_friend[2]['friends']}, followrs={min_friend[2]['followers']}"
            )
            return min_friend[1]
        except:
            min_friend = min(future_friends)
            logger.info(
                f"try retweeting/fav https://twitter.com/i/status/{min_friend[1]} from potential friend profile: {min_friend[2]['id_str']} friends= {min_friend[2]['friends']}, followrs={min_friend[2]['followers']}"
            )
            return min_friend[1]
    else:
        logger.info(
            f"try retweeting from original post: https://twitter.com/i/status/{tweet_id}"
        )
        return tweet_id


def filter_tweet(search_results: list, twitter_api):
    """

    function to ensure that retweets are on-topic
    by the hashtag list

    Args:
        search_results:
        twitter_api:
        flag:

    Returns:

    """
    filtered_search_results = []

    for status in search_results:

        faved_sum = (
            status.retweet_count,
            status.favorite_count,
            status.retweet_count + status.favorite_count,
        )

        if status.is_quote_status:
            try:
                quoted_tweet = twitter_api.get_status(
                    status.quoted_status_id_str, tweet_mode="extended"
                )

            except tweepy.TweepError as e:
                telegram_bot_sendtext(f"ERROR {e}, twitter.com/anyuser/status/{status.id_str}")
                quoted_tweet = ""
                continue

            end_status = get_longest_text(status) + get_longest_text(quoted_tweet)
        else:
            end_status = get_longest_text(status)

        if len(end_status.split()) > 3 and faved_sum[2] > 1:

            joined_list = Settings.add_hashtag + Settings.retweet_include_words

            # remove elements from the exclude words list
            keyword_matches = [
                x
                for x in joined_list + Settings.watch_add_hashtag
                if x in end_status.lower()
                and not any(
                    [
                        x
                        for x in Settings.retweet_exclude_words
                        if x in end_status.lower()
                    ]
                )
            ]

            if keyword_matches:

                if any(
                    [x for x in keyword_matches if x not in Settings.watch_add_hashtag]
                ):
                    print(keyword_matches, status.full_text)

                    filtered_search_results.append(
                        (faved_sum, status.id_str, status.full_text)
                    )
                else:
                    logger.info(f">> skipped, {keyword_matches}, {end_status}")


    return sorted(filtered_search_results)


def try_give_love(twitter_api, in_tweet_id, self_followers):
    """
    try to favorite a post from simple users
    Args:
        twitter_api:
        in_tweet_id:
        self_followers:

    Returns:

    """
    # todo add flag to use sleep or fav immediately

    tweet_id = find_simple_users(twitter_api, in_tweet_id, self_followers)

    if not is_in_logfile(in_tweet_id, Settings.faved_tweets_output_file):

        try:
            time.sleep(randint(0, 250))
            twitter_api.create_favorite(id=tweet_id)
            write_to_logfile({in_tweet_id: {}}, Settings.faved_tweets_output_file)
            _status = twitter_api.get_status(tweet_id)
            json_add_user(_status.author.id_str)
            message_log = (
                "faved tweet succesful: https://twitter.com/i/status/{}".format(
                    tweet_id
                )
            )
            logger.info(message_log)
            telegram_bot_sendtext(message_log)

            return True

        except tweepy.TweepError as e:
            if e.api_code in Settings.IGNORE_ERRORS:
                write_to_logfile({in_tweet_id: {}}, Settings.faved_tweets_output_file)
                logger.debug(f"throw a en error {e}")
                logger.exception(e)
                telegram_bot_sendtext(f"{e}")
                return False
            else:
                logger.error(e)
                telegram_bot_sendtext(f"{e}")
                return True

    else:
        logger.info("Already faved (id {})".format(tweet_id))


def fav_or_tweet(max_val, flag, twitter_api):
    """

    use a tweet or a fav function depending on the flag called
    Args:
        max_val:
        flag:
        twitter_api:

    Returns:

    """

    self_followers = get_followers_list()
    count = 0

    while count < len(max_val):

        tweet_id = max_val[-1 - count][1]
        tweet_text = max_val[-1 - count][2]
        logger.info(f"{len(tweet_text.split())}, {tweet_text}")

        if flag == "give_love":
            use_function = try_give_love(twitter_api, tweet_id, self_followers)
            log_message = "fav"

        else:
            use_function = try_retweet(
                twitter_api, tweet_text, tweet_id, self_followers
            )
            log_message = "retweet"

        if use_function:
            logger.info(f"{log_message}ed: id={tweet_id} text={tweet_text}")
            break
        else:
            count += 1
            time.sleep(2)
            if count >= len(max_val):
                logger.debug("no more tweets to post")
            continue


def search_and_retweet(flag: str = "global_search", count: int = 100):
    """
    Search for a query in tweets, and retweet those tweets.

    Args:
        flag: A query to search for on Twitter. it can be `global_search` to search globally
              or `list_search` reduced to a list defined on mylist_id
        count: Number of tweets to search for. You should probably keep this low
               when you use search_and_retweet() on a schedule (e.g. cronjob)

    Returns: None

    """
    try:
        twitter_api = twitter_setup()
        if flag == "global_search":
            # search results retweets globally forgiven keywords
            search_results = twitter_api.search(
                q=get_query(), count=count, tweet_mode="extended"
            )  # standard search results
        elif flag == "list_search":
            # search list retwwets most commented ad rt from the experts lists
            search_results = twitter_api.list_timeline(
                list_id=Settings.mylist_id, count=count, tweet_mode="extended"
            )  # list to tweet from


        elif flag == "give_love":
            search_results = twitter_api.list_timeline(
                list_id=Settings.mylist_id, count=count, tweet_mode="extended"
            ) + twitter_api.list_timeline(
                list_id=1396081589768138755, count=count, tweet_mode="extended"
            )

    except tweepy.TweepError as e:
        logger.exception(e.reason)
        telegram_bot_sendtext(f"ERROR: {e}, {search_results[0]}")
        return
    except Exception as e:
         telegram_bot_sendtext(f"ERROR: {e}")

    # get the most faved + rtweeted and retweet it
    max_val = filter_tweet(filter_repeated_tweets(search_results, flag), twitter_api)
    fav_or_tweet(max_val, flag, twitter_api)


def retweet(tweet: tweepy.Status):
    """
    re-tweet self last tweeted message.
    Args:
        tweet: tweet object

    Returns: None

    """

    try:
        twitter_api = twitter_setup()

        if not hasattr(tweet, 'retweeted'):
            print(tweet)
            twitter_api.retweet(id=tweet.id)
            logger.info(f"retweeted: twitter.com/i/status/{tweet.id}")
            telegram_bot_sendtext(f"Self retweeted: twitter.com/drugscibot/status/{tweet.id}")

        else:
            twitter_api.unretweet(tweet.id)
            twitter_api.retweet(tweet.id)
            telegram_bot_sendtext(f"Self re-retweeted: twitter.com/drugscibot/status/{tweet.id}")

    except tweepy.TweepError as e:
        logger.exception(e)
        telegram_bot_sendtext(f"ERROR:{e}")


def retweet_old_own():
    """

    Returns: None

    """

    twitter_api = twitter_setup()

    with open(Settings.posted_urls_output_file, "r") as jsonFile:
        article_log = json.load(jsonFile)

    article_log_reversed = {article_log[x]['tweet_id']:{**article_log[x], **{'id':x}} for x in article_log}


    min_val = min(article_log[x]["count"] for x in article_log)

    for art in sorted(list(article_log_reversed), key=None, reverse=False):
        tweet = twitter_api.statuses_lookup([article_log_reversed[art]["tweet_id"]])
        if tweet and article_log_reversed[art]["count"] <= min_val:
            retweet(tweet[0])
            article_log[article_log_reversed[art]['id']]["count"] += 1

            break

    with open(Settings.posted_urls_output_file, "w") as fp:
        json.dump(article_log, fp, indent=4)


def display_help():
    """
    Show available commands.

    Returns: Prints available commands

    """

    print("Syntax: python {} [command]".format(sys.argv[0]))
    print()
    print(" Commands:")
    print("    rss    Read URL and post new items to Twitter")
    print("    rtg    Search and retweet keywords from global feed")
    print("    rtl    Search and retweet keywords from list feed")
    print("    glv    Fav tweets from list or globally")
    print("    rto    Retweet last own tweet")
    print("    sch    Run scheduled jobs on infinite loop")
    print("    help   Show this help screen")


if __name__ == "__main__":
    main()



#!/usr/bin/python3.6
from setuptools import setup

setup(
    name="scienceBot",
    version="0.1.1.1",
    description="Bot for sci-com and policy-com ",
    url="https://github.com/fanasal/science_bot",
    license="GNU Affero General Public License v3.0",
    packages=["scibot"],
    keywords=[
        "psychedelics",
        "fact-checking",
        "sci-com",
        "drug policy",
        "research",
        "science",
    ],
    entry_points={
        "console_scripts": [
            "scibot=scibot.what_a_c:main",
        ]
    },
    install_requires=[
        "beautifulsoup4==4.9.3",
        "feedparser==6.0.2",
        "oauthlib==3.1.1",
        "python-dotenv==0.15.0",
        "requests==2.26",
        "requests-oauthlib==1.3.0",
        "schedule==0.6.0",
        "telebot==0.0.4",
        "Telethon==1.18.2",
        "tweepy==4.4.0",
        "urllib3==1.26.7",
    ],
    zip_safe=False,
)



#!/usr/bin/python3.6
import logging
import json
import re
import os
import time
import datetime
import feedparser
import dateutil.parser
from os.path import expanduser
from scibot.telebot import telegram_bot_sendtext
from scibot.streamer import listen_stream_and_rt
from schedule import Scheduler

# logging parameters
logger = logging.getLogger("bot logger")
# handler determines where the logs go: stdout/file
file_handler = logging.FileHandler(f"{datetime.date.today()}_scibot.log")

logger.setLevel(logging.DEBUG)
file_handler.setLevel(logging.DEBUG)

fmt_file = (
    "%(levelname)s %(asctime)s [%(filename)s: %(funcName)s:%(lineno)d] %(message)s"
)

file_formatter = logging.Formatter(fmt_file)
file_handler.setFormatter(file_formatter)
logger.addHandler(file_handler)


class Settings:
    """Twitter bot application settings.

    Enter the RSS feed you want to tweet, or keywords you want to retweet.
    """

    IGNORE_ERRORS = [327, 139]
    # RSS feeds to read and post tweets from.
    feed_urls = [
        "https://pubmed.ncbi.nlm.nih.gov/rss/search/1X9MO_201KJGQLdG05NdxtaqKjTZuIPIGlgpiDZr31QjkgZUbj/?limit=300&utm_campaign=pubmed-2&fc=20210922175019",
        "https://pubmed.ncbi.nlm.nih.gov/rss/search/1XSES1Yl3kEgnfOg6EStNFyWMogtYXic2VVXS8rpsyNHTjv1HK/?limit=200&utm_campaign=pubmed-2&fc=20210510224301",
        "https://pubmed.ncbi.nlm.nih.gov/rss/search/1jAe3RzQKmf7SOUEM-Dt7QQtMWNG2UffuIIo_GGKHPfoKqhY9f/?limit=200&utm_campaign=pubmed-2&fc=20210510224348",
        "https://pubmed.ncbi.nlm.nih.gov/rss/search/1bCr63ThlO22Eg5TxBaIQ5mzH02TqtmtM1QIkqa66iqK4SsMJm/?limit=200&utm_campaign=pubmed-2&fc=20210510224551",
        "https://pubmed.ncbi.nlm.nih.gov/rss/search/1hEma6JdH30sOOO0DiTP1jZh-6ZgoypoEsw_B9tXZejk_E8QuX/?limit=200&utm_campaign=pubmed-2&fc=20210510230918",
    ]
    # rss best results no time harm reduction and psychedelics
    feed_older_literature = feedparser.parse("https://pubmed.ncbi.nlm.nih.gov/rss/search/1h_Yu2rLTrK0AIYDN2V5HLWSksLTr4a6SUZjZzoAPcf-Qk0gCJ/?limit=200&utm_campaign=pubmed-2&fc=20210901021150")["entries"]

    pre_combined_feed = [feedparser.parse(url)["entries"] for url in feed_urls]

    # (combined_feed)

    combined_feed = [item for feed in pre_combined_feed for item in feed]
    combined_feed.sort(
        key=lambda x: dateutil.parser.parse(x["published"]), reverse=True
    )

    # Log file to save all tweeted RSS links (one URL per line).
    posted_urls_output_file = expanduser("~/drugscibot/publications.json")

    # Log file to save all retweeted tweets (one tweetid per line).
    posted_retweets_output_file = expanduser("~/drugscibot/posted-retweets.log")

    # Log file to save all retweeted tweets (one tweetid per line).
    faved_tweets_output_file = expanduser("~/drugscibot/faved-tweets.log")

    # Log file to save followers list.
    users_json_file = expanduser("~/drugscibot/users.json")

    # Include tweets with these words when retweeting.
    retweet_include_words = [
        "drugpolicy",
        "drugspolicy",
        "transformdrugspolicy",
        "transformdrugpolicy",
        "drugchecking",
        "regulatestimulants",
        "regulatedrugs",
        "sensibledrugpolicy",
        "drugpolicyreform",
        "safeconsumption",
        "harmreduction",
        "druguse",
        "regular",
        "reduccion de dano",
        "dosis minima",
        "regulacion",
        "droga",
        "sicoactiva",
        "psicoactiva",
        "politica de droga",
        # "cion de riesgo",
        "legalizacion",
        "safesuply",
        "safersuply",
    ]

    # Do not include tweets with these words when retweeting.
    retweet_exclude_words = [
        "sex",
        "sexual",
        "sexwork",
        "sexualwork",
        "fuck",
        "vaping",
        "vape",
        "cigarretes",
        "nicotine",
        "smoke",
        "smoking",
        "constellationsfest",# to be deleted after the festival
        "zigaretten",
    ]

    add_hashtag = [
        "psilocybin",
        "psilocybine",
        "psychedelic",
        "hallucinogenic",
        "overdose",
        "microdosing",
        "drug-policy",
        "drugspolicy",
        "mdma",
        "drug checking",
        "drugpolicy",
        "drug policy",
        "ayahuasca",
        "psychopharmacology",
        "neurogenesis",
        "5-meo-dmt",
        "serotonergic",
        "ketamine",
        "psychotherapy",
        "harm reduction",
        "methadone",
    ]  # trip

    # do not retweet if search results include only a single of these keywords
    watch_add_hashtag = [
        "alzheimer",
        "depression",
        "anxiety",
        "dmt",
        "droga",
        "lsd",
        "therapy",
        "psychiatry",
        "mentalhealth",
        "trip",
        "regula",
        "regular",
        "mental health",
        "clinical trial",
        "consciousness",
        "meta-analysis",
        "dopamine",
        "serotonin",
        "psychological",
        "metaanalysis",
        "reform",
    ]

    # list of the distribution
    mylist_id = "1306244304000749569"


class SafeScheduler(Scheduler):
    """
    An implementation of Scheduler that catches jobs that fail, logs their
    exception tracebacks as errors, optionally reschedules the jobs for their
    next run time, and keeps going.
    Use this to run jobs that may or may not crash without worrying about
    whether other jobs will run or if they'll crash the entire script.
    """

    def __init__(self, reschedule_on_failure=True):
        """

        Args:
            reschedule_on_failure: if is True, jobs will be rescheduled for their
        next run as if they had completed successfully. If False, they'll run
        on the next run_pending() tick.
        """
        self.reschedule_on_failure = reschedule_on_failure
        super().__init__()

    def _run_job(self, job):
        try:
            super()._run_job(job)

        except Exception as e:
            logger.exception(e)
            telegram_bot_sendtext(f"[Job Error] {e}")
            job.last_run = datetime.datetime.now()
            job._schedule_next_run()


def shorten_text(text: str, maxlength: int) -> str:
    """
    Truncate text and append three dots (...) at the end if length exceeds
    maxlength chars.

    Args:
        text: The to shorten.
        maxlength: The maximum character length of the text string.

    Returns: Shortened text string.

    """
    return (text[:maxlength] + "...") if len(text) > maxlength else text


def insert_hashtag(title: str) -> str:
    """
    Add hashtag on title for keywords found on Settings.add_hashtag
    Args:
        title: Text to parse for inserting hash symbols

    Returns: Text with inserted hashtags

    """

    for x in Settings.add_hashtag:
        if re.search(fr"\b{x}", title.lower()):
            pos = (re.search(fr"\b{x}", title.lower())).start()
            if " " in x:
                title = title[:pos] + "#" + title[pos:].replace(" ", "", 1)
            else:
                title = title[:pos] + "#" + title[pos:]
    return title


def compose_message(item: feedparser.FeedParserDict) -> str:
    """
    Compose a tweet from an RSS item (title, link, description)
    and return final tweet message.

    Args:
        item: feedparser.FeedParserDict
        An RSS item

    Returns: mMssage suited for a Twitter status update.

    """
    title = insert_hashtag(item["title"])

    message = shorten_text(title, maxlength=250) + " 1/5  " + item["link"]
    return message


def is_in_logfile(content: str, filename: str) -> bool:
    """
    Does the content exist on any line in the log file?

    Args:
        content: Content to search file for.
        filename: Full path to file to search.

    Returns: `True` if content is found in file, otherwise `False`.

    """
    if os.path.isfile(filename):
        with open(filename, "r") as jsonFile:
            article_log = json.load(jsonFile)
        if content in article_log:
            return True
    return False


def write_to_logfile(content: dict, filename: str) -> None:
    """
    Append content to json file.

    Args:
        content: Content to append to file
        filename: Full path to file that should be appended.

    Returns: None

    """
    try:
        with open(filename, "w") as fp:
            json.dump(content, fp, indent=4)
    except IOError as e:
        logger.exception(e)


def scheduled_job(read_rss_and_tweet, retweet_own, search_and_retweet):

    # listen_stream_and_rt('#INSIGHT2021')

    schedule = SafeScheduler()
    # job 1
    schedule.every().day.at("22:20").do(read_rss_and_tweet)
    schedule.every().day.at("06:20").do(read_rss_and_tweet)
    schedule.every().day.at("14:20").do(read_rss_and_tweet)
    # job 2
    schedule.every().day.at("01:10").do(retweet_own)
    schedule.every().day.at("09:10").do(retweet_own)
    schedule.every().day.at("17:10").do(retweet_own)
    # job 3
    schedule.every().day.at("00:20").do(search_and_retweet, "list_search")
    schedule.every().day.at("03:20").do(search_and_retweet, "list_search")
    schedule.every().day.at("06:20").do(search_and_retweet, "list_search")
    schedule.every().day.at("09:20").do(search_and_retweet, "list_search")
    schedule.every().day.at("12:20").do(search_and_retweet, "list_search")
    schedule.every().day.at("15:20").do(search_and_retweet, "list_search")
    schedule.every().day.at("18:20").do(search_and_retweet, "list_search")
    schedule.every().day.at("21:20").do(search_and_retweet, "list_search")

    schedule.every().day.at("01:25").do(search_and_retweet, "list_search")
    schedule.every().day.at("04:25").do(search_and_retweet, "list_search")
    schedule.every().day.at("07:25").do(search_and_retweet, "list_search")
    schedule.every().day.at("10:25").do(search_and_retweet, "list_search")
    schedule.every().day.at("13:25").do(search_and_retweet, "list_search")
    schedule.every().day.at("16:25").do(search_and_retweet, "list_search")
    schedule.every().day.at("19:25").do(search_and_retweet, "list_search")
    schedule.every().day.at("22:25").do(search_and_retweet, "list_search")
    # job love
    schedule.every(5).minutes.do(search_and_retweet, "give_love")

    while 1:
        schedule.run_pending()
        time.sleep(1)
